!module parameters 
        
        
!end module parameters
MODULE nrtype
        INTEGER, PARAMETER :: I4B = SELECTED_INT_KIND(9)
        INTEGER, PARAMETER :: I2B = SELECTED_INT_KIND(4)
        INTEGER, PARAMETER :: I1B = SELECTED_INT_KIND(2)
! SP kind changed to "double precisions."
        INTEGER, PARAMETER :: SP = KIND(1.0d0)
        INTEGER, PARAMETER :: DP = KIND(1.0D0)
        INTEGER, PARAMETER :: SPC = KIND((1.0,1.0))
        INTEGER, PARAMETER :: DPC = KIND((1.0D0,1.0D0))
        INTEGER, PARAMETER :: LGT = KIND(.true.)
        REAL(SP), PARAMETER :: PI=3.141592653589793238462643383279502884197_sp
        REAL(SP), PARAMETER :: PIO2=1.57079632679489661923132169163975144209858_sp
        REAL(SP), PARAMETER :: TWOPI=6.283185307179586476925286766559005768394_sp
        REAL(SP), PARAMETER :: SQRT2=1.41421356237309504880168872420969807856967_sp
        REAL(SP), PARAMETER :: EULER=0.5772156649015328606065120900824024310422_sp
        REAL(DP), PARAMETER :: PI_D=3.141592653589793238462643383279502884197_dp
        REAL(DP), PARAMETER :: PIO2_D=1.57079632679489661923132169163975144209858_dp
        REAL(DP), PARAMETER :: TWOPI_D=6.283185307179586476925286766559005768394_dp
        TYPE sprs2_sp
                INTEGER(I4B) :: n,len
                REAL(SP), DIMENSION(:), POINTER :: val
                INTEGER(I4B), DIMENSION(:), POINTER :: irow
                INTEGER(I4B), DIMENSION(:), POINTER :: jcol
        END TYPE sprs2_sp
        TYPE sprs2_dp
                INTEGER(I4B) :: n,len
                REAL(DP), DIMENSION(:), POINTER :: val
                INTEGER(I4B), DIMENSION(:), POINTER :: irow
                INTEGER(I4B), DIMENSION(:), POINTER :: jcol
        END TYPE sprs2_dp
END MODULE nrtype

!------------------------------------------------------------------------------- 
program confidence
	use nr, only: sort

	implicit none
	integer :: i
	integer :: k
        integer :: nyears
        integer, parameter :: n_sim=100000
        real, allocatable, dimension(:) :: t_jja
        real, allocatable, dimension(:) :: e_t_jja
        real, allocatable, dimension(:) :: e_t_jja_resample
        real, allocatable, dimension(:) :: t_jja_sim
        real :: mittel_t_jja
        real, allocatable, dimension(:) :: jahr
        real :: a_fit_t_jja
        real :: b_fit_t_jja
        real, allocatable, dimension(:) :: a_fit_t_jja_sim
        real, allocatable, dimension(:) :: b_fit_t_jja_sim
        real :: b_neu_99
	real :: b_neu_95
	real :: b_neu_90
        real :: b_neu_10
	real :: b_neu_5
	real :: b_neu_1
	integer :: iocheck
	integer :: open_error
	

	open (unit=10, file='daten.txt', status='old',                      & !Bestimmung der Datenanzahl
              form='formatted', action='read', iostat=open_error)
        read(10,*) 
        i = 1
        do while (.true.)
           read (10, *, iostat = iocheck) t_jja
           if (iocheck .ne. 0) exit
           i = i + 1
        end do
        close (unit=10, status='keep')
        nyears = i - 1
        	
	allocate(t_jja(nyears))						     !Besetzen der Felder
	allocate(e_t_jja(nyears))
	allocate(e_t_jja_resample(nyears))
	allocate(t_jja_sim(nyears))
	allocate(jahr(nyears))
	allocate(a_fit_t_jja_sim(n_sim))
	allocate(b_fit_t_jja_sim(n_sim))	
	
	open(10,file='daten.txt')					     !Auslesen der Klimadaten
	read(10,*)
	write(*,*) 'Daten auslesen...'
	do i=1,nyears
	read(10,*) t_jja(i)
	end do
	close(10)	
	
	mittel_t_jja=sum(t_jja)/nyears					    !Berechnung des Mittelwertes
	
	write(*,*) 'Residuen berechnen...'
	do i=1,nyears
		e_t_jja(i)=t_jja(i)-mittel_t_jja		   	    !Berechnung der Residuen
		jahr(i)=i+1892.0
	end do
	
	write(*,*) 'Regression durchfuehren...'
	call fit(jahr,t_jja,nyears,a_fit_t_jja,b_fit_t_jja)      	    !lineare Regression      
	
	write(*,*) 'Simulation...'
	do k=1,n_sim 							    !Begin Simulation
	call bootm(e_t_jja,nyears,1,e_t_jja_resample)
	
	do i=1,nyears
	t_jja_sim(i)=mittel_t_jja+e_t_jja_resample(i)
	end do
	
	call fit(jahr,t_jja_sim,nyears,a_fit_t_jja_sim(k),b_fit_t_jja_sim(k)) 
	
	
 	end do 	    							    !Ende Simulation
 	
 	write(*,*) 'Sortieren...'
 	write(*,*)
 	write(*,*)
 	write(*,*)
 	
 	call sort(b_fit_t_jja_sim)					    !Sortieren der simulierten Trendparameter
 	
 	b_neu_99=b_fit_t_jja_sim(nint(0.99*n_sim))			    !Berechnung der Perzentilen
 	b_neu_95=b_fit_t_jja_sim(nint(0.95*n_sim))
 	b_neu_90=b_fit_t_jja_sim(nint(0.90*n_sim))
 	
 	b_neu_1=b_fit_t_jja_sim(nint(0.01*n_sim))
 	b_neu_5=b_fit_t_jja_sim(nint(0.05*n_sim))
 	b_neu_10=b_fit_t_jja_sim(nint(0.1*n_sim))
 	
 	write(*,*) 'Datenanzahl=',nyears
 	 	
 	open(20,file='ausgabe.txt')					    !Ausgabe der Daten
 	write(20,*) 'Datenazahl=',nyears
 	write(*,'(a9,2x,f10.7)') 'original:',b_fit_t_jja
 	if(b_fit_t_jja.lt.0.0) then
 	write(*,'(3x,a6,2x,f10.7)') 'b_99%:',b_neu_1
 	write(*,'(3x,a6,2x,f10.7)') 'b_95%: ',b_neu_5
 	write(*,'(3x,a6,2x,f10.7)') 'b_90%: ',b_neu_10
 	if(b_fit_t_jja.le.b_neu_1) then
 	write(*,*) 'ueber dem 99% Konfidenzband'
 	end if
 	if(b_fit_t_jja.le.b_neu_5) then
 	write(*,*) 'ueber dem 95% Konfidenzband'
 	end if
 	if(b_fit_t_jja.le.b_neu_10) then
 	write(*,*) 'ueber dem 90% Konfidenzband'
 	end if
 	end if
 	if(b_fit_t_jja.ge.0.0) then 	
 	write(*,'(3x,a6,2x,f10.7)') 'b_99%:',b_neu_99
 	write(*,'(3x,a6,2x,f10.7)') 'b_95%: ',b_neu_95
 	write(*,'(3x,a6,2x,f10.7)') 'b_90%: ',b_neu_90
 	if(b_fit_t_jja.ge.b_neu_99) then
 	write(*,*) 'ueber dem 99% Konfidenzband'
 	end if
 	if(b_fit_t_jja.ge.b_neu_95) then
 	write(*,*) 'ueber dem 95% Konfidenzband'
 	end if
 	if(b_fit_t_jja.ge.b_neu_90) then
 	write(*,*) 'ueber dem 90% Konfidenzband'
 	end if
 	end if
  	
 	write(20,'(a9,2x,f10.7)') 'original:',b_fit_t_jja
 	if(b_fit_t_jja.lt.0.0) then
 	write(20,'(3x,a6,2x,f10.7)') 'b_99%:',b_neu_1
 	write(20,'(3x,a6,2x,f10.7)') 'b_95%: ',b_neu_5
 	write(20,'(3x,a6,2x,f10.7)') 'b_90%: ',b_neu_10
 	if(b_fit_t_jja.le.b_neu_1) then
 	write(20,*) 'ueber dem 99% Konfidenzband'
 	end if
 	if(b_fit_t_jja.le.b_neu_5) then
 	write(20,*) 'ueber dem 95% Konfidenzband'
 	end if
 	if(b_fit_t_jja.le.b_neu_10) then
 	write(20,*) 'ueber dem 90% Konfidenzband'
 	end if
 	end if
 	if(b_fit_t_jja.ge.0.0) then 	
 	write(20,'(3x,a6,2x,f10.7)') 'b_99%:',b_neu_99
 	write(20,'(3x,a6,2x,f10.7)') 'b_95%: ',b_neu_95
 	write(20,'(3x,a6,2x,f10.7)') 'b_90%: ',b_neu_90
 	if(b_fit_t_jja.ge.b_neu_99) then
 	write(20,*) 'ueber dem 99% Konfidenzband'
 	end if
 	if(b_fit_t_jja.ge.b_neu_95) then
 	write(20,*) 'ueber dem 95% Konfidenzband'
 	end if
 	if(b_fit_t_jja.ge.b_neu_90) then
 	write(20,*) 'ueber dem 90% Konfidenzband'
 	end if
 	end if
  
 	close(20)
 	
 	deallocate (t_jja,e_t_jja,e_t_jja_resample)
 	deallocate (t_jja_sim,jahr,a_fit_t_jja_sim,b_fit_t_jja_sim)
 	
 	write(*,*)
 	write(*,*)
 	write(*,*)
 	write(*,*)
 	write(*,*) 'Beenden mit Enter!'
 	pause
 	
 	
end program confidence

! ======================================================================
!     Berechnung des Moving Block Bootstrap (MBB) fuer e(i,j) bzw. r(i,j)
subroutine bootm(e,n,l_mbb,esim)
!        use nrtype
!        use nr, only:ran
!        use parameters 
!	use ran        
         
        implicit none
        integer, parameter :: nyears=113
	real, dimension(n), intent(in) :: e		!Residuen (Vektor)
	integer, intent(in) :: n			!Anzahl der Daten
	integer, intent(in) :: l_mbb
	real :: ran
     			!Blocklaenge
!        real, dimension(1:nyears,1:seventhree), intent(out) :: esim_matrix  ! resampelte Residuen (Matrix)
        
      	real, dimension(n) :: esim			  	  !resampelte Residuen in der Subroutine (Vektor)
!      	logical, dimension(1:seventhree,1:nyears) :: maske 	  !Maskenmatrix fuer unpack (selbe Dimension wie esim_matrix)
!      	real, dimension(1:seventhree,1:nyears) :: esm
      	integer, dimension(n) ::  indxx
      	integer :: idum=1
      	real :: r7						 
      
      	integer i,j,k
      	integer nbs
!      	open(8,file='ausgabe.txt')	

      	nbs=n-l_mbb+1
      
      	if (l_mbb.eq.1) then
           do i=1,n
              r7=ran(idum)				                       	
              indxx(i)=int(n*r7)+1
           end do
      	else
           k=1
outer:     do i=1,n
              r7=ran(idum)				      	
              indxx(k)=int(nbs*r7)+1
              k=k+1
              if (k.gt.n) exit outer
              do j=1,l_mbb-1
                 indxx(k)=indxx(k-1)+1
                 k=k+1
                 if (k.gt.n) exit outer
              end do
           end do outer
        end if
      
      do i=1,n					
          esim(i)=e(indxx(i))    	
      end do

!      maske=.true.            			
!      esm=unpack(esim,maske,esm)			!sim. Residuenvektor ---> sim. Residuenmatrix     
!      esim_matrix=transpose(esm)
      
!       write(8,'(113(f5.1,1x))') esim_matrix
!      close(8)

      return      

end subroutine bootm

	FUNCTION ran(idum)
	IMPLICIT NONE
	INTEGER, PARAMETER :: K4B=selected_int_kind(9)
	INTEGER(K4B), INTENT(INOUT) :: idum
	REAL :: ran
	INTEGER(K4B), PARAMETER :: IA=16807,IM=2147483647,IQ=127773,IR=2836
	REAL, SAVE :: am
	INTEGER(K4B), SAVE :: ix=-1,iy=-1,k
	if (idum <= 0 .or. iy < 0) then
		am=nearest(1.0,-1.0)/IM
		iy=ior(ieor(888889999,abs(idum)),1)
		ix=ieor(777755555,abs(idum))
		idum=abs(idum)+1
	end if
	ix=ieor(ix,ishft(ix,13))
	ix=ieor(ix,ishft(ix,-17))
	ix=ieor(ix,ishft(ix,5))
	k=iy/IQ

	iy=IA*(iy-k*IQ)-IR*k
	if (iy < 0) iy=iy+IM
	ran=am*ior(iand(IM,ieor(ix,iy)),1)
	END FUNCTION ran
!===================================================================================================
!               Subroutine zur Berechnung der zeitl. veraenderl. Mittelwerte y=a+bx
SUBROUTINE fit(x,y,n,a,b)
!       Geaendert: chi2, q und sig herausgenommen
	USE nrtype   !; USE nrutil, ONLY : assert_eq
!	USE nr, ONLY : gammq
	IMPLICIT NONE
	integer, intent(in) :: n
	REAL, DIMENSION(n), INTENT(IN) :: x,y
	REAL, INTENT(OUT) :: a,b
!	REAL(SP), DIMENSION(:), OPTIONAL, INTENT(IN) :: sig
!	INTEGER(I4B) :: ndata
	REAL :: sigdat,ss,sx,sxoss,sy,st2
	REAL, DIMENSION(n) :: t
	a=0.0
	b=0.0
		ss=real(n,sp)
		sx=sum(x)
		sy=sum(y)
	sxoss=sx/ss
	t(:)=x(:)-sxoss
		b=dot_product(t,y)
	st2=dot_product(t,t)
	b=b/st2
	a=(sy-sx*b)/ss
!	siga=sqrt((1.0_sp+sx*sx/(ss*st2))/ss)
!	sigb=sqrt(1.0_sp/st2)
 END SUBROUTINE fit
 
 subroutine plotgnu
	call system ('wgnuplot.exe gnuskript.plt')
!	call system ('test.plt ')
 end subroutine plotgnu     
 

!include 'C:\Karsten\Diplom\Potsdam\Fortran\MonteCarloSim\nr.f90' 
!include 'C:\Dokumente und Einstellungen\kuerbis\Desktop\Diplom\Potsdam\Fortran\MonteCarloSim\nrutil.f90'
